/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          ep.ui.dialog
 *
 * @revision        $Revision: 1.1.2.1.2.1 $
 */

// load ep.alert, because it is used in CKEditor plugin epPageBreak
jQuery.plugin( 'ep.ui.previeweditor', [ 'ep.ui.core', 'jQuery.ui.widget', 'jQuery.ckeditor.adapters.jquery', 'ep.alert'], function( $, undefined ){

    var editorStack = {};
    /*
    {
        'Text' : {
            'editors' : [node1,node2], //jQuery objects of editor nodes with this particular name
            'toolbar' : node,
            'leftEditor' : $Object,
            'rightEditor' : $Object,
            'disabledLayer': $Object
            }
    }
    */

    var toolbarContainer = ep('<div>')
                .prependTo('body')
                .attr('id','uieditor_preview_toolbar')
                .hide();

    var top$ = jQuery,
        topCKEDITOR = CKEDITOR;

    try {
        if(top.jQuery){
            top$ = top.jQuery;
        }
        //top.CKEDITOR = topCKEDITOR = CKEDITOR;
    }
    catch (error) {
    }

    // on window resize -> adjust width of editors to avoid display errors
    $(window).on('resize',function(){
        // move to another thread
        setTimeout(function(){
            // loop over editor instances
            for(var key in CKEDITOR.instances){
                var instance = CKEDITOR.instances[key],
                    iframe = jq(instance.container.$).find('iframe');

                // set possible width
                iframe.width('100%');
            }
        }, 1);
    });

    // register method to manage availability of toolbar elements
    CKEDITOR.manageToolbar = function(disabled){
        var toolbarElements = top$('#EditorTools *[data-ep-uieditor], #EditorTools select');

        //toolbarElements[!!disabled ? 'addClass' : 'removeClass']('Disabled');
        toolbarElements.toggleClass('Disabled', !!disabled);
    };

    // define ckeditor on parent frame
    //top.CKEDITOR = CKEDITOR;

    $.widget('ep.ui.previeweditor',{
        editorInstance : undefined,
        toolbarId : 'uieditor_preview_toolbar',
        options: {
        //  name: undefined,
        //  navbar: undefined,
            pageBreak: false,
            holder: undefined,
            base: undefined,
            height: 300,
            width: 400
        },

        _create: function(){
            var self = this,
                o = self.options,
                elem = self.element,
                editorName = o.name = o.name ? o.name : 'editor_' + new Date().getTime();

            self.id = elem.attr('id');

            // create new stack entry if necessary
            if(!editorStack[editorName]){
                editorStack[editorName] = {};
            }

            // register toolbar in stack
            editorStack[editorName].toolbar = $('#' + self.toolbarId);
            // create editor
            self._createEditor();
        },

        _createEditor: function(){
            var self = this,
                o = self.options,
                elem = self.element;

            // create editor
            elem.ckeditor(function(textarea){
                self.editorInstance = elem.ckeditorGet();
//#JSCOVERAGE_IF false
// commented out for jscoverage because it is just relevant for the preview with an external toolbar in a higher leveled iframe
                // set flag to identify if PageBreak icon has to be shown in external toolbar
                self.editorInstance.pageBreak = !!o.pageBreak;

                // register textarea for undo
                if(typeof $$ !== "undefined"){
                    var undoObject = $$('undoWidget').getUndoObject(),
                        undoNode = $('<input type="hidden">').appendTo('body').attr('name', elem.attr('name'));

                    undoNode.val(elem.val());
                    undoObject.registerInput(undoNode[0]);

                    //only change ckeditor content when input was changed via undo/redo call
                    undoNode.on('change',function(evt){
                        if(top.epages.isUndoCall){
                            self.editorInstance.setData(undoNode[0].value);
                        }
                    });
                }

                // set default value for source flag
                self.editorInstance.hasSource = false;

                // handler if editor gets the focus
                self.editorInstance.focusManager.focus = function(evt){
                    self.toggleToolbar(true);
                    self.editorInstance.fire('focus');
                    // manage visibility of the two external toolbars
                    top.jq('#PageTools').addClass('NavBarInvisible').removeClass('NavBarVisible');
                    top.jq('#EditorTools').addClass('NavBarVisible').removeClass('NavBarInvisible');

                    // show/hide PageBreak icon
                    top.jq('#EditorTools .MCEElementsBox-InsertPageBreak')[self.editorInstance.pageBreak? 'removeClass' : 'addClass']('HideElement');

                    // enable/disable toolbar buttons (depending on source code mode)
                    CKEDITOR.manageToolbar(self.editorInstance.hasSource);
                };

                // handler if editor loses the focus
                self.editorInstance.focusManager.blur = function(evt){
                    // update textarea
                    self.editorInstance.updateElement();
                    self.toggleToolbar(false);

                    // if ckeditor instance is in source mode -> change mode and reset source flag
                    if(self.editorInstance.hasSource){
                        self.editorInstance.execCommand('source');
                        self.editorInstance.hasSource = false;
                        CKEDITOR.manageToolbar(self.editorInstance.hasSource);
                        self.editorInstance.preventTwice = true;
                    }

                    //check if something was changed inside the editor and write it back to the hidden input for undo handling
                    if(undoNode && undoNode.val() != elem.val()){
                        undoNode.val(elem.val());
                        epages.event.fire(undoNode[0],'change');
                    }
                };

                //check change content on ckeditor undo handling and add it to epages undo handling
                self.editorInstance.on('saveSnapshot',function(e){
                    if(undoNode && undoNode.val() != elem.val()){
                        undoNode.val(elem.val());
                        epages.event.fire(undoNode[0],'change');
                    }
                });

                self.toggleToolbar(false);

                // handling to mark toolbar elements if the current selection has changed
                self.editorInstance.on('selectionChange',function(ev){
                    var pathElements = ev.data.path.elements,
                        toolbarCommands = top$('#EditorTools *[data-ep-uieditor]'),
                        commandsArray = [],  // takes all commands (handed over by classes)
                        pathArray = [],      // takes tag names (lowercase) of all path elements
                        styles = {},         // takes all the relevant styles
                        styleTupel,
                        styleArray,
                        styleKey;

                    // loop over all elements depending on the selection
                    for(var key in pathElements){
                        var idKey = pathElements[key].$.tagName.toLowerCase(),
                            attributes = pathElements[key].$.attributes[0];

                        // check if path element has an attributes and style property
                        if(attributes !== undefined){
                            // if a style is set
                            if(attributes.name === "style"){
                                // split style definitions
                                styleArray = attributes.value.substr(0, attributes.value.length-1).split(";");

                                // loop over style definitions
                                for(var styleCount=0, styleCountLength=styleArray.length; styleCount < styleCountLength; styleCount++){
                                    // split value (e.g. [font-size]['9px']
                                    styleTupel = styleArray[styleCount].split(":");

                                    // generate camel-case key (e.g. font-size -> fontSize)
                                    styleKey = styleTupel[0].replace(/\-(\w)/g, function (strMatch, strBracket, strPos, strSrc){
                                        return strBracket.toUpperCase();
                                    });

                                    // set style definition only from the deepest selection
                                    if(styles[styleKey] === undefined){
                                        switch(styleKey){
                                            // indent -> data-ep-uieditor="indent"
                                            case 'marginLeft':
                                                styles[styleKey] = $.merge(['indent', idKey], styleTupel);
                                                break;
                                            // color -> data-ep-uieditor="fontcolor"
                                            case 'color':
                                                styles[styleKey] = $.merge(['fontcolor', idKey], styleTupel);
                                                break;
                                            // backgroundColor -> data-ep-uieditor="backgroundColor"
                                            case 'backgroundColor':
                                                styles[styleKey] = $.merge(['backgroundColor', idKey], styleTupel);
                                                break;
                                            // default -> data-ep-uieditor="$.trim(styleTupel[1])"
                                            default:
                                                styles[styleKey] = $.merge([$.trim(styleTupel[1]), idKey], styleTupel);
                                                break;
                                        }
                                    }
                                }
                            }
                        }

                        // push tag name in array
                        pathArray.push(idKey);
                        // get element's class name, concatenate it with tag name and push in array
                        var className = pathElements[key].$.className.toLowerCase();
                        if(className){
                            idKey += '_' + className;
                        }
                        commandsArray.push(idKey);
                    }

                    // loop over all toolbar elements
                    toolbarCommands.each(function(i,elem){
                        var dataValue = $(elem).data('epUieditor');

                        // select/mark or reset element
                        if(elem.tagName.toLowerCase() == 'option'){
                            $(elem).closest('select').val((jQuery.inArray($(elem).data('epUieditor'),pathArray) != -1)? $(elem).data('epUieditor') : '');
                        }
                        else{
                            $(elem)[jQuery.inArray(dataValue,commandsArray) != -1 ? 'addClass' : 'removeClass']('Selected');
                        }
                    });

                    // loop over registered style properties
                    for(var _key in styles){
                        // get associated toolbar element
                        var elem = top$('#EditorTools *[data-ep-uieditor="'+styles[_key][0]+'"]');

                        // select or mark element
                        if(elem[0].tagName.toLowerCase() == 'option'){
                            elem.closest('select').val(elem.val());
                        }
                        else{
                            elem.addClass('Selected');
                            // if an element exists to display the current color
                            if(elem.find('div.ColorBox').length){
                                elem.find('div.ColorBox').css('background-color', styles[_key][3]);
                            }
                        }
                        delete styles[_key];
                    }

                    // set default for text-align when nothing is selected
                    if($.grep(['right', 'center', 'justify'], function(value, index){
                        return top$('#EditorTools *[data-ep-uieditor="'+value+'"]').hasClass('Selected');
                    }).length == 0){
                        top$('#EditorTools *[data-ep-uieditor="left"]').addClass('Selected');
                    }

                });

                elem.on('change',function(){
                    self.editorInstance.setData();
                });

//#JSCOVERAGE_ENDIF

                // get iframe from editor instance
                var ckeIframe = $(self.editorInstance.container.$).find('iframe');

                // add class to iframe to set special CSS for preview inline mode and remove inline styles build by CKEditor
                ckeIframe
                    .addClass('cke_preview')
                    .removeAttr('style')
                    .parent()
                    .removeAttr('style');

                // add class to the html element inside the iframe to set special CSS for preview inline mode
                ckeIframe
                    .contents()
                    .find('html')
                    .addClass(function(){
                        var addedClass = 'cke_preview';
                        if(o.navbar){
                            addedClass += ' cke_' + o.navbar;
                        }
                        return addedClass;
                    });

                // hide base div if defined
                if(o.base){
                    o.base.hide();
                }

                // show editor
                if(o.holder){
                    o.holder
                        .height(o.height+10)
                        .width(o.width)
                        .show();
                }

                // set initial dimensions
                ckeIframe
                    .height(o.height) //(o.height+10)
                    .width(o.holder? "100%" : o.width);

                // set focus to editor
                self.editorInstance.focusManager.focus();
                // set cursor in editor
                self.setCursor(self.editorInstance, CKEDITOR.POSITION_BEFORE_START); // POSITION_AFTER_END

                // save initial width
                self.editorInstance.initWidth = o.width;
                $(window).trigger('resize');
            },
            //options for editor
            {
                sharedSpaces :
                {
                    top : 'uieditor_preview_toolbar'
                },
                toolbar: o.pageBreak ? 'FullPageBreak' : 'Full'
            });
        },

        // set cursor to editor
        // position can be: POSITION_AFTER_END, POSITION_BEFORE_START
        // call: self.setCursor(self.editorInstance, CKEDITOR.POSITION_BEFORE_START);
        setCursor: function (editorInstance, position) {
            editorInstance.focus();

            var selection = editorInstance.getSelection(),
                range = selection.getRanges()[0],
                paragraph,
                newRange;

            if (range) {
                paragraph = (range.startContainer.getName && range.startContainer.getName() === "body") ? range.startContainer : range.startContainer.getAscendant({p: 2}, true);
                newRange = new CKEDITOR.dom.range(range.document);

                if (newRange.endOffset !== null) {
                    newRange.moveToPosition(paragraph, position);
                    newRange.select();
                }
            }
        },

        toggleToolbar: function(show){
            try{
                if(show && this.editorInstance){
                    // if ckeditor instance changes and old instance is in source mode -> change mode and reset source flag
                    if(typeof top.EP_CKEDITOR !== "undefined" && top.EP_CKEDITOR != this.editorInstance && top.EP_CKEDITOR.hasSource){
//#JSCOVERAGE_IF false
                        top.EP_CKEDITOR.execCommand('source');
                        top.EP_CKEDITOR.hasSource = false;
//#JSCOVERAGE_ENDIF
                    }

                    top.EP_CKEDITOR = this.editorInstance;
                }
            }
            catch(e){
                jQuery.error(e);
            }

            return this;
        },

        destroy: function(){
            var self = this,
                id = self.id;

            // destroy according ckeditor instance
            CKEDITOR.instances[id].destroy();

            self.Inherited(arguments);
        }
    });
});