/*
 * @copyright   © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module      de_epages.presentation.ui.tray
 *
 * @revision    $$
 */
jQuery.plugin( 'de_epages.presentation.ui.tray', {
	plugin: [
		'de_epages.core',
		'ep.ui.dialog',
		'backbone',
		'jQuery.tmpl',
		'ep.ajax',
		'ep.dict',
		'jQuery.metaparse',
		'jQuery.ui.tabs',
		'jQuery.ui.selectable'
	],
	tmpl: [
		'de_epages.presentation.ui.tray-view' // Templates von Views bzw. GadgetsDialog laden lassen?
	]
}, function( $, undefined ){
		/*
		* @dictionary				ep.dict
		*
		* @translation			{Apply}
												{Products}
												{Customers}
												{Tray}
												{NoResults}
												{FirstPage}
												{LastPage}
												{Next}
												{Back}
												{Page}
												{of}
												{CountItems}
												{DeleteSearch}
												{StartSearch}
	 */

		/*
		* @dictionary				de_epages.presentation.dictionary
		*
		* @translation			{AddFromTray}
												{AddElementsFromTray}
												{AddElementsFromTrayInfoTextInsert}
												{AddElementsFromTrayInfoTextEmpty}
												{TrayIsEmpty}
		*/
	"use strict";
	var dict = ep.dict('ep.dict'),
		moduleDict = ep.dict('de_epages.presentation.dictionary'),
		loadedData = {}, //currently all tray instances share the same data, will be changed with EPG-25667
		currentSearchString = '',
		tablePageSize = 10,
		serverPagerPageSize = 50;

/*### request handler for data handling of all tray instances ###*/
	var requestHandler = function(options){
		var doRequest = false,
			pagerPage = 1;
		//add basic request params these are equal for every request
		var requestData = [
			{'name' : 'ObjectID','value': ep.config.siteId},
			{'name' : 'ViewAction', 'value' : 'JSONViewResponse'},
			{'name' : 'PageSize', 'value' : serverPagerPageSize},
			{'name' : 'WildSearchString', 'value': currentSearchString},
			{'name' : 'GetTableHead', 'value' : 1}
		];
		//changeAction has to be defined for paging in tables
		if(options.changeAction){
			requestData.push({'name' : 'ChangeAction', 'value' : options.changeAction});
		}
		//if no changeAction is defined we add all
		else{
			$.each(options.searchActions,function(i,value){
				requestData.push({'name': 'ChangeAction','value' : value});
			});
		}
		if(!options.sortColumn){
			//default sort is Alias
			options.sortColumn = 'Alias';
		}
		requestData.push({'name' : 'OrderBy', 'value' : options.sortColumn});
		if(options.sortOrder){
			requestData.push({'name' : 'OrderDesc', 'value' : 1});
		}
		else{
			options.sortOrder = 0;
			requestData.push({'name' : 'OrderDesc', 'value' : 0});
		}
		if(options.id && options.page){
			var currentData = loadedData[options.id];
			if(currentData && currentData.aaData[options.page] //check if requested Data is already loaded used for piplining
				&& currentData.sOrderBy == options.sortColumn     //sort Order has to be the same, sorting is done by server
				&& currentData.sOrderDes == options.sortOrder){
				var returnValue = {
					'aaData' : currentData.aaData[options.page],
					'aoColumns' : currentData.aoColumns,
					'iTotalDisplayRecords' : currentData.iTotalRecords,
					'iTotalRecords' : currentData.iTotalRecords,
					'sEcho' : new Date().getTime()
				};
				//add Sorting to data
				var sortData;
				if(options.sortColumn){
					$.each(currentData.aoColumns,function(columnInd,columnElem){
						if(columnElem.sName && columnElem.sName == options.sortColumn){
							sortData = [[columnInd,"asc"]];
						}
					});
				}
				if(sortData){
					returnValue['aaSorting'] = sortData;
				}
				else{
					returnValue['aaSorting'] = [[1,"asc"]];
				}
				if(options.callback){
					options.callback(returnValue);
				}
			}
			else{
				//calculate server pager page
				pagerPage = Math.ceil(options.page * tablePageSize / serverPagerPageSize);
				requestData.push({'name' : 'Page', 'value' : pagerPage});
				doRequest = true;
			}
		}
		else{
			doRequest = true;
		}
		if(doRequest){
			ep.ajax({
				'type':'POST',
				'dataType': 'json',
				'data': requestData
			}).done(function(data, textStatus, jqXHR){
				$.each(data,function(ind,elem){
					//create table pages with defined size
					// Pages from server are larger to avoid too many request
					var pages = {};
					if(elem.aaData.length){
						var pageStartCount = ((pagerPage - 1) * (serverPagerPageSize / tablePageSize)) + 1;
						var pageEndCount = Math.ceil(elem.aaData.length / tablePageSize) + pageStartCount -1;
						for(var i = pageStartCount; i <= pageEndCount;i++){
							pages[i] = elem.aaData.splice(0,tablePageSize);
						}
					}
					else{
						pages['1'] = [];
					}
					//check if current request extends already cached data or adds new data
					if(loadedData[ind] && loadedData[ind].sOrderBy == options.sortColumn && loadedData[ind].sOrderDes == options.sortOrder){
						$.extend(loadedData[ind].aaData,pages);
					}
					else{
						//Some requests do not send order data, so we have to add this data in order to have it in the same schema
						if(!elem.sOrderDes){
							elem.sOrderDes = options.sortOrder;
						}
						if(!elem.sOrderBy){
							elem.sOrderBy = options.sortColumn;
						}
						//add request data to cache and create pages to aaData -> used for datatables.js
						$.each(elem.aoColumns,function(ind, val){
							elem.aoColumns[ind]['sName'] = val.sTitle;
							if(val.sDisplayTitle) {
								elem.aoColumns[ind]['sTitle'] = val.sDisplayTitle;
							} else {
								elem.aoColumns[ind]['sTitle'] = moduleDict.translate(val.sTitle);
							}
						});
						loadedData[ind] = elem;
						loadedData[ind].aaData = pages;
					}
					currentData = loadedData[ind];
				});

				if(!options.saveButton){
				var returnValue = {
						//'aaData' : options.saveButton ? data : ($.isEmptyObject( currentData.aaData ) ? currentData.aaData : currentData.aaData[options.page ? options.page : 1]),
					'aaData' : $.isEmptyObject( currentData.aaData ) ? currentData.aaData :currentData.aaData[options.page ? options.page : 1],
					'aoColumns' : currentData.aoColumns,
					'iTotalDisplayRecords' : currentData.iTotalRecords,
					'iTotalRecords' : currentData.iTotalRecords,
					'sEcho' : new Date().getTime()
				};
				var sortData;
				if(options.sortColumn){
					$.each(currentData.aoColumns,function(columnInd,columnElem){
						if(columnElem.sName && columnElem.sName == options.sortColumn){
							sortData = [[columnInd,"asc"]];
						}
					});
				}
				}
				else{
					var returnValue = {
						'aaData' : data
					}
				}
				//add sorting parameters to return for datatable
				if(sortData){
					returnValue['aaSorting'] = sortData;
				}
				else{
					returnValue['aaSorting'] = [[1,"asc"]];
				}
				returnValue['aaSorting'] = [[1,"asc"]];
				if(options.callback){
					options.callback(returnValue);
				}
			});
		}
	};
/*### end of request handler ###*/


	var searchBox = (function ($, Backbone) {
		var view = Backbone.View.extend({
			dialogView : undefined,
			template: 'de_epages.presentation.ui.tray-view',
			selectedIds: {},
			events : {
				'click button' : 'startSearch',
				'keydown input' : 'select'
			},

			initialize : function () {
				var self = this;
				self.$el = self.options.node;
				self.render();

				// suche Submit Button aus umschliessenden Formular
				var submitButton = self.$el.find('input').closest('form').find('button[type=submit]');

				submitButton.on('click', function(evt){
					// get input node
					if(!self.options.inputNode){
						self.options.inputNode = self.$el.find('input');
					}
					// get value of input node
					currentSearchString = self.options.inputNode.val();

					if(currentSearchString.length == ""){
						return;
					}

					// prevent submit
					evt.preventDefault();

					// execute the request
					requestHandler({
						'searchActions' : self.options.searchActions,
						'saveButton': true,
						'callback': function(data){
							// get tray data and its length
							var aaData = data.aaData,
								obj = {},
								countResults = 0;

							// loop over result object to get the number of results
							$.each(aaData, function(index, value){
								if(Number(value.iTotalRecords) > 0 && (value.aaData[1])){
									countResults += value.aaData[1].length;
									obj[index] = [];
									obj[index].push(value.aaData[1][0][0]);
									return false;
								}
							});

							// check how many results was found
							switch(countResults){
								case 0:
									self.options.inputNode.closest('form').trigger('submit');  // no result -> just submit the form
									break;
								case 1:
									// if a callback was sumitted and only one result was found
									if(self.options.callback){
										self.options.callback({
											'objectIds': obj,
											'insertFromTray' : false,
											'additionalContent': undefined
										});
									}
									break;
								default:
									// open dialog for more than one result
									self.startSearch();
							}
						}
					});
				});
			},

			render : function () {
				var self = this;
				self.$el.html($.tmpl(self.template,{'inputName': self.options.inputName}));
				self.$el
					.dictParse('ep.dict', true)
					.dictParse('de_epages.presentation.dictionary', true)
					.find('.ep-js')
					.metaparse();
				return self;
			},

			select : function (evt) {
				if(evt.keyCode == $.ui.keyCode.ENTER){
					evt.preventDefault();
					evt.stopPropagation();
					this.startSearch();
				}
			},

			startSearch: function(){
				var self = this;
				//in some cases the object tray can be disabled so we do nothing in this case
				if(!this.$el.hasClass('Disabled')){
					if(!self.options.inputNode){
						self.options.inputNode = self.$el.find('input');
					}
					currentSearchString = self.options.inputNode.val();
					loadedData = {};
					self.selectedIds = {};
					//get data from server for all defined searchActions
					requestHandler({
						'searchActions' : self.options.searchActions,
						'callback': function(data){
							//check if data from server contains only one result
							//in this case we directly call the defined callback and do not open
							//the dialog
							self.showResults();
						}
					});
				}
			},

			doSearch: function(searchString){
				var self = this;
				self.options.inputNode.val(searchString);
				self.startSearch();
			},

			dialogApply: function(data){
				var self = this;
				if(self.options.callback){
					self.options.callback({
						'objectIds': self.dialogView.getSelectedIds(),
						'insertFromTray' : data.insertFromTray,
						'additionalContent': data.additionalContent
					});
				}
			},

			showResults: function(){
				var self = this;
				if(!self.dialogView){
					self.dialogView = new DialogView(self.options)
						.on('doSearch',self.doSearch,self)
						.on('apply',self.dialogApply,self);
				}
				self.dialogView.render();
			}

		});
	return view;
	})(jQuery, Backbone);

	var DialogView = (function ($, Backbone) {
		"use strict";
		var view = Backbone.View.extend({
			tagName : 'div',
			template : 'de_epages.presentation.ui.tray-dialogview',
			trayTemplate: 'de_epages.presentation.ui.tray-tabpaneltray',
			tabContentViews: undefined,
			tabsNode: undefined,
			activeTabId: undefined,

			initialize: function(){
				var self = this;
					self.dialogNode = ep(self.$el).uiDialog({
						width : 900,
						height : 600,
						modal : false,
						autoOpen: false,
						title: moduleDict.translate('AddFromTray'),
						close: function(event, ui){
							self.options.inputNode.val("");
						},
						buttons : [{
							text : dict.translate('Apply'),
							click : function() {
								ep(this).uiDialog("close");
								if(self.options.callback){
									var useTrayNode = self.dialogNode.find('.ep-trayUseTray');
									self.trigger('apply',{
										'insertFromTray': useTrayNode && useTrayNode.attr('checked') ? true : false,
										'additionalContent' : self.dialogNode.find('.ep-trayAditionalContent')
									});
								}
							}
						}]
					});
			},

			render : function () {
				var self = this;
				self.tabContentViews = [];
				self.dialogNode.empty();

				$.ready({ tmpl:self.template}, function(){
					self.dialogNode
						.append($.tmpl(self.template))
						.find('.ep-js')
							.metaparse()
							.end()
						.uiDialog('open');
						//add tab view for every defined search action type (Products,Categories, etc)
						self.tabsNode = self.dialogNode.find('.ep-trayTabs');
						$.each(self.options.searchActions,function(searchId,value){
							self.tabsNode.tabs('add','#ep-tray' + searchId + '-tab',moduleDict.translate(searchId) + '<span id="ep-tray' + searchId + '-tabName"></span>');
							var newTab = self.dialogNode.find('#ep-tray' + searchId + '-tab'),
								tabContent = new TabContentView({
								id: searchId,
								node: newTab,
								additionalContent: self.options.additionalContent,
								searchActions: self.options.searchActions
							});
							tabContent.on('doSearch',self.doSearch,self);
							self.tabContentViews.push(tabContent);
						});

						//add tray - show tray as extra tab which only show the content of the current tray
						if(self.options.useTray){
							self.tabsNode.tabs('add','#ep-trayTray-tab',dict.translate('Tray'));
							self.tabsNode.tabs({
								'select': function(evt,ui){
									var nextPanel = $(ui.panel);
									//create tray content on selection of tray tab to avoid useless request for tray content if it is not used
									if(nextPanel.attr('id') === 'ep-trayTray-tab'){
										if(nextPanel.children().length === 0){
											$.ready({ tmpl:self.trayTemplate}, function(){
												ep.ajax({
													type : 'POST',
													dataType: 'json',
													data : {
														'ObjectID' : ep.config.siteId,
														'ViewAction' : 'JSONGetTray'
													}
												})
												.done(function(data){
													nextPanel
														.html($.tmpl(self.trayTemplate,{'groups': data}))
														.dictParse('ep.dict', true)
														.dictParse('de_epages.presentation.dictionary', true)
														.find('.ep-js')
															.metaparse();
												});
											});
										}
									}
								}
							});
						}
						if(self.activeTabId){
							self.tabsNode.tabs('option','selected',self.activeTabId);
						}
				});
				return this;
			},

			doSearch: function(searchString){
				//trigger event "doSearch" on current Element to trigger Search on Main view but keep views separated
				var self = this;
				self.activeTabId = self.tabsNode.tabs('option','selected');
				this.trigger('doSearch',searchString);
			},

			getSelectedIds: function(){
				var self = this,
					selectedIds = {};
				$.each(self.tabContentViews,function(index, value){
					$.extend(selectedIds,value.getSelectedIds());
				});
				return selectedIds;
			}
		});
		return view;
	})(jQuery, Backbone);

	var TabContentView = (function ($, Backbone) {
		"use strict";
		var view = Backbone.View.extend({
			tagName : 'div',
			template : 'de_epages.presentation.ui.tray-tabcontentview',
			tabContent: undefined,
			events : {
				'click .ep-trayStartSearch' : 'doSearch',
				'click .ep-traySearchDelete' : 'clearSearch',
				'keyup input': 'keyUpInput'
			},

			initialize:function(){
				this.render();
			},

			render : function () {
				var self = this;
				$.ready({ tmpl:self.template}, function(){
					self.$el = self.options.node;
					self.$el
						.append($.tmpl(self.template,{searchString: currentSearchString}))
						.dictParse('ep.dict', true);
					//add additional content
					if(self.options.additionalContent){
						var additionalContentNode = self.$el.find('.ep-trayAditionalContent');
						additionalContentNode
							.html(self.options.additionalContent)
								.find('.ep-js')
									.metaparse();
					}
					var contentNode = self.$el.find('.ep-trayPageContent');
					self.tabContent = new ListView({
						id: self.options.id,
						node: contentNode,
						searchActions: self.options.searchActions
					});
					//handling for input clear button (X)
					if(!currentSearchString){
						self.$el.find('.ep-traySearchDelete').hide();
					}
					self.delegateEvents(self.events);
				});
				return this;
			},

			keyUpInput: function(evt){
				//handling for input clear button (X)
				var deleteSearchString = this.$el.find('.ep-traySearchDelete');
				if($(evt.currentTarget).val().length){
					deleteSearchString.show();
				}
				else{
					deleteSearchString.hide();
				}
				//start search on enter
				if(evt.keyCode == $.ui.keyCode.ENTER){
					this.doSearch();
				}
			},

			doSearch: function(evt){
				this.trigger('doSearch',this.$el.find('.ep-traySearch').val());
			},

			clearSearch: function(){
				this.trigger('doSearch','');
			},

			getSelectedIds: function(){
				var self = this,
					selectedIds = {};
				selectedIds[self.options.id] = self.tabContent.getSelectedIds();
				return selectedIds;
			}
		});
		return view;
	})(jQuery, Backbone);

	//this is where the magic happens, data is setup for datatable widget
	var ListView = (function ($, Backbone) {
		"use strict";
		var view = Backbone.View.extend({
			tagName : 'div',
			selectedIds: undefined,

			initialize:function(){
				this.render();
			},

			render : function () {
				var self = this;
				self.selectedIds = [];
				$.ready({
					plugin:['jQuery.datatables']
				}, function(){
					self.$el = self.options.node;
					var currentId = self.options.id;
					//get data from cache or server only for this page of the table and this current type (products etc)
					requestHandler({
						'searchActions': self.options.searchActions,
						'id': currentId,
						'page': 1,
						'callback': function(data){
							//add parameters to define the datatable
							var additionalTableData = {
								//change the first colum to show checkboxes because it is always the colum with the objectids
								"aoColumnDefs": [
									{
										"aTargets": [ 0 ],
										"bSortable": false,
										"mRender": function ( data, type, full ) {
											return '<input type="checkbox" value="' + data + '" class="ep-js" data-js="ep.uiInput()" />';
										}
									}
								],

								//change table layout to ePages UI
								"sDom": 'prt<"PagerFooter"<"ep-trayPageCount">p<"ep-trayObjectCount">>',
								//add select all checkbox to header after header render event
								"fnHeaderCallback": function( nHead, aData, iStart, iEnd, aiDisplay ) {
									$(nHead).find('th:first').html('<input type="checkbox" value="" class="ep-js" data-js="ep.uiInput()" />');
									$(nHead).find('th').each(function(){
										var $elem = $(this);
										if($elem.hasClass('sorting_asc')){
											$elem.find('.list_ico_s_changeorder_up').removeClass('HideElement');
										}
										else{
											$elem.find('.list_ico_s_changeorder_up').addClass('HideElement');
										}
										if($elem.hasClass('sorting_desc')){
											$elem.find('.list_ico_s_changeorder_down').removeClass('HideElement');
										}
										else{
											$elem.find('.list_ico_s_changeorder_down').addClass('HideElement');
										}
										if($elem.hasClass('sorting')){
											$elem.find('.list_ico_s_changeorder_unselected').removeClass('HideElement');
										}
										else{
											$elem.find('.list_ico_s_changeorder_unselected').addClass('HideElement');
										}
									});

									$(nHead).find('input[type="checkbox"]').on('click',function(){
										if($(this).attr('checked')){
											self.$el.find('tbody tr.RowSelected').trigger('click');
										}
										else{
											self.$el.find('tbody tr:not(.RowSelected)').trigger('click');
										}
									});
								},

								//add selectbox change handling after table is initialized
								"fnInitComplete" : function( oSettings, json ){
									$(this).find('th').each(function(){
										var $elem = $(this);
										$elem
											.append('<span class="LeftMargin ep-sprite list_ico_s_changeorder_up HideElement"></span>'
											+'<span class="LeftMargin ep-sprite list_ico_s_changeorder_down HideElement"></span>'
											+'<span class="LeftMargin ep-sprite list_ico_s_changeorder_unselected HideElement"></span>');
										if($elem.hasClass('sorting_asc')){
											$elem.find('.list_ico_s_changeorder_up').removeClass('HideElement');
										}
										if($elem.hasClass('sorting_desc')){
											$elem.find('.list_ico_s_changeorder_down').removeClass('HideElement');
										}
										if($elem.hasClass('sorting')){
											$elem.find('.list_ico_s_changeorder_unselected').removeClass('HideElement');
										}
									});
									this.on('click','input[type="checkbox"],tr',function(evt){
										//if checkbox is clicked check if it is selected or deselected and add or remove the object id of the selectedIds array
										var row = $(this).closest('tr');
										if(row.attr('role') === undefined){
											row.toggleClass('RowSelected');
											var checkBox = row.find('input[type="checkbox"]');
											checkBox.attr('checked',row.hasClass('RowSelected') ? true : false);
											var objectId = checkBox.val();
											if(row.hasClass('RowSelected')){
												if($.inArray( objectId, self.selectedIds) === -1){
													self.selectedIds.push(objectId);
												}
											}
											else{
												var arrayIndex = $.inArray( objectId, self.selectedIds);
												if(arrayIndex !== -1){
													self.selectedIds.splice(arrayIndex, 1);
												}
											}
											//update selected count on tab name
											var countString = self.selectedIds.length ? ' (' + self.selectedIds.length + ')' : '';
											$('#ep-tray' + currentId + '-tabName').html(countString);
											evt.stopPropagation();
										}
									});
								},

								//draw callback is called after every page draw
								//do metaparse, checked already selected checkboxes and correct footer
								"fnDrawCallback": function( oSettings ) {
									self.$el.find('.ep-js').metaparse();
									self.$el.find('.ep-trayPageCount')
										.html(
											dict.translate('Page') +' '+ (Math.ceil(oSettings._iDisplayStart / 10) + 1) +' ' + dict.translate('of') + ' ' + Math.ceil(oSettings._iRecordsTotal / 10));
									self.$el.find('.ep-trayObjectCount').html(dict.translate('CountItems') +': ' + oSettings._iRecordsTotal);
									self.$el.find('input[type="checkbox"]').each(function(i,val){
										var checkBox = $(val);
										if($.inArray( checkBox.val(), self.selectedIds) !== -1){
											checkBox.attr('checked',true);
										}
									});
								},
								"sPaginationType": "full_numbers",
								//Processing and Serverside need to be true to activate our data handling
								"bProcessing": true,
								"bServerSide": true,
								"bAutoWidth" : false,
								"sAjaxSource": "?",
								//add translation for our pager design
								"oLanguage": {
									"sZeroRecords": dict.translate('NoResults'),
									"oPaginate": {
										"sFirst" : '<img title="' + dict.translate('FirstPage') + '" src="' + ep.config.iconsRoot + '/list_ico_s_paging_first.png" />',
										"sPrevious" : '<img title="' + dict.translate('Back') + '" src="' + ep.config.iconsRoot + '/list_ico_s_paging_backward.png" />',
										"sNext" : '<img title="' + dict.translate('Next') + '" src="' + ep.config.iconsRoot + '/list_ico_s_paging_forward.png" />',
										"sLast" : '<img title="' + dict.translate('LastPage') + '" src="' + ep.config.iconsRoot + '/list_ico_s_paging_last.png" />'
									}
								},
								//this function is called everytime the table needs new data - changes page
								"fnServerData": function ( sSource, aoData, fnCallback, oSettings ) {
									var pageNumber = (oSettings._iDisplayStart / oSettings._iDisplayLength) + 1;
									requestHandler({
										'searchActions': self.options.searchActions,
										'id': self.options.id,
										'page': pageNumber,
										'sortColumn': oSettings.aoColumns[oSettings.aaSorting[0][0]].sName,
										'sortOrder': oSettings.aaSorting[0][2],
										'callback':	fnCallback
									});
								}
							};
							//add loaded data to the object which is used to initialize the table
							$.extend(data, additionalTableData);
							 self.$el.find('table').dataTable(data);
						}
					});
				});
				return this;
			},

			getSelectedIds: function(){
				return this.selectedIds;
			}
		});
		return view;
	})(jQuery, Backbone);
		//add tray to de_epages.presentation.ui to be able to create a tray
		$.extend( $.scope('de_epages.presentation.ui'), {
		tray: function (options) {
			new searchBox(options);
			/*possible options
			* node - DOM node on which the tray builds the mainview (input and button)
			* inputName - string for the input, is used for the form if the customer inserts a string in the input and saves page
			* searchActions': { - object with search actions which are sent to the server and provides data for the tables
				'Customers': 'JSONSearchCustomersToInsert' - key will be used as identifier inside tray and for translation of tab naem
																									- value is the changeAction which provides the data in the correct form for the tray
			},
			additionalContent - html string, this string is added to every tab page of dialog, content gets metaparsed, will be given back to callback
			useTray - boolean value , if true tab tray is added to the dialog
			callback - function which gets data after object tray is closed or a single value was found
				data for callback
				data: {
					objectIds: {	- object with arrays of selected object ids for every defined SearchAction
						Customers: []
					},
					insertFromTray - boolean value, true if user selected the checkbox on the tray page
					additionalContent - dom node with additional content which was given to the function, checkboxes can be checked for change etc
			*/
		}
	});
});