/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.editor
 *
 * @revision		$Revision: 1.1.2.1.2.3 $
 */

jQuery.plugin( 'ep.ui.editor', [ 'ep.ui.core', 'jQuery.ui.widget', 'jQuery.ckeditor.adapters.jquery','jQuery.event.special.dom', 'ep.fn.busy'], function( $, undefined ){

	var editorStack = {};
	/*  // example for an entry in editorStack
	{
		'Text' : {
			'editors' : [node1,node2], //jQuery objects of editor nodes with this particular name
			'toolbar' : node,
			'leftEditor' : $Object,
			'rightEditor' : $Object,
			'disabledLayer': $Object
			}
	}
	*/

	// recalculate toolbar width if window is resized
	$(window).on('resize.uiEditor',function(){
		for(var key in editorStack){
			var editors = editorStack[key].editors;
			// iterate over "language" specific editors
			for(var i=editors.length-1; i>=0; i--){
				ep(editors[i]).uiEditor('adjustToolbar',key);
				ep(editors[i]).uiEditor('reloadToolbar',key);
			}
		}
	});

	$.widget('ep.ui.Editor', {
	//	editorInstance : undefined,
		baseFloatZIndex: 450,

		options: {
		//	name: undefined,
		//	toolbar: undefined,
			hideToolbarOnStartup: false,
			noToolbar : false,
			useAbsolute: false,
			disable: false,
			blogPostMode: false,
			showPreview: false,
			enterMode: 1 // is the same as CKEDITOR.ENTER_P
		},

		_create: function(){
			var self = this,
				o = self.options,
				elem = self.element,
				editorName = o.name = o.name ? o.name : 'editor_' + new Date().getTime();

			// push editor element in stack
			if(editorStack[editorName]){
				editorStack[editorName].editors.push(elem);
			}
			else{
				editorStack[editorName] = { 'editors' : [elem] };
				var toolbarContainer = ep('<div>')
					.insertBefore(elem)
					.attr('id','ep-toolbarContainer'+editorName)
					.hide();

				editorStack[editorName].toolbar = toolbarContainer;
			}

			// if editor element is visible -> create editor instance immediately
			if(elem.is(':visible')){
				self._createEditor();
				elem.parent().on('show',function(){
					self.adjustToolbar(editorName);
				});
			}
			// else: create instance when editor element becomes visible
			else{
				elem.one('show',function(){
					self._createEditor();
					elem.parent().on('show',function(){
						self.adjustToolbar(editorName);
					});
				});
			}
		},

		// method to create editor instance
		_createEditor: function(){
			var self = this,
				o = self.options,
				elem = self.element,
				editorName = o.name,
				stack = editorStack[editorName],
				// additional configuration options for the editor which will be built below (2nd argument for the .ckeditor method)
				editorOptions = {
					enterMode: o.enterMode,
					sharedSpaces:{
						top : 'ep-toolbarContainer'+editorName
					}
				};

			// Pass options for plugins to the editor instance.
			o.pluginOptions && $.extend(editorOptions, {pluginOptions : o.pluginOptions});

			if(o.toolbar){
				$.extend(editorOptions, {toolbar: o.toolbar});
			}
			// indicates whether to use a absolute/relative url in linkpicker widget/plugin
			if(o.useAbsolute){
				$.extend(editorOptions, {useAbsolute: o.useAbsolute});
			}

			if(o.showPreview){
				$.extend(editorOptions, {coreStyles_italic: { element : 'i', overrides : 'em' }});
				CKEDITOR.config.showPreview = o.showPreview;
			}

			var toolbarMode = 'Full';
			if(o.blogPostMode){
					toolbarMode += 'PageBreak';
					$.extend(editorOptions, {blogPostMode: true});
			}
			if ($('#ep-toolbarContainer'+editorName).closest('tr').width() <= 1080){
					toolbarMode = toolbarMode.replace('Full','Mini');
			}
			// After the first resize the help sidebar could be expanded and resize the editor width
			$(window).one('resize.uiEditor',function(){
				if ($('#ep-toolbarContainer'+editorName).closest('tr').width() <= 1080){
					toolbarMode = toolbarMode.replace('Full','Mini');
					$.extend(editorOptions, {toolbar: toolbarMode});
				}
			});
			$.extend(editorOptions, {toolbar: toolbarMode
			//	, startupMode: 'source'
			});

			elem.ckeditor(function(textarea){
				// hide busy layer
				ep(elem.closest('div')).busy('hide');

				self.editorInstance = elem.ckeditorGet();
				self.baseFloatZIndex = self.editorInstance.config.baseFloatZIndex;

				var snapshotOnceEvent = function(){
					elem.trigger('change');
					self.editorInstance.removeListener('saveSnapshot',snapshotOnceEvent);
				};
				self.editorInstance.on('saveSnapshot',snapshotOnceEvent);

				//callback function after editor is ready
				var firstEditorNode =  $(stack.editors[0].ckeditorGet().container.$),
					currentEditorNode = $(self.editorInstance.container.$),
					leftEditor = firstEditorNode,
					rightEditor = firstEditorNode;

				// iterate over "language" specific editors -> find the most left and most right editor
				for(var i = 0, iLength = stack.editors.length; i < iLength; i++){
					var editorNode = $(stack.editors[i].ckeditorGet().container.$),
						offsetLeft = editorNode.offset().left;

					if(offsetLeft < leftEditor.offset().left){
						leftEditor = editorNode;
					}
					else if(offsetLeft > rightEditor.offset().left){
						rightEditor = editorNode;
					}
				}
				stack.leftEditor = leftEditor;
				stack.rightEditor = rightEditor;

				// calculate toolbar width
				self.adjustToolbar(editorName,true);

				// if toolbar should not be shown on start up
				if(o.hideToolbarOnStartup){
					self.editorInstance.focusManager.focus = function(){
						self.toggleToolbar(true);
						self.editorInstance.fire('focus');
					};
					self.editorInstance.focusManager.blur = function(){
						self.toggleToolbar(false);
					};
					self.toggleToolbar(false);
				}

				// trigger window resize to adjust toolbars of all created editors
				$(window).trigger('resize.uiEditor', [true]);

				// update corresponding textarea when losing focus
				self.editorInstance.on('blur',function(){
					self.editorInstance.updateElement();
				});

				// write data when content changed
				elem.on('change', function(evt){
					self.editorInstance.setData();
					self.editorInstance.focus();
				});

				// if disable layer is undefined -> append new one to element and add to stack
				if(stack.disabledLayer === undefined){
					stack.disabledLayer = $("<div />");
					stack.disabledLayer.insertBefore(stack.toolbar);
				}

				// style disable layer (element lies over editor and toolbar => both cannot be used)
				stack.disabledLayer
					.addClass('cke_full_disabled')
					.css({
//						position: 'absolute',
//						top: 0,
//						left: 0,
//						opacity: 0.7,
//						backgroundColor: '#fff',
						zIndex: stack.toolbar.css('z-index')+1,
						display: 'none',
						width: stack.toolbar.width() + "px",
						height: (stack.toolbar.height() + $(self.editorInstance.container.$).height()) + "px"
					});

				//
				// In case we never display a toolbar and the editor is disabled, we go into *AlmostFullscreen*
				// mode for editing on click onto the *disabledLayer*.
				//
				o.disable && o.hideToolbarOnStartup && o.noToolbar && stack.disabledLayer.on('click', $.proxy(function (event) {
					this.execCommand('epAlmostFullscreen');
				}, self.editorInstance));

				// remove title attributes from dropdowns (do not show obvious tooltips)
				stack.toolbar.find('.cke_rcombo a').removeAttr('title');

				// set initial state for disable layer
				self.disableEditor(o.disable);

				$(window).trigger('scroll');
			},
			//options for editor
			editorOptions);
		},

		// show/hide disable layer
		disableEditor: function(disable){
			editorStack[this.options.name].disabledLayer.css('display', (disable ? 'block' : 'none'));
		},

		// calculate new width of toolbar and resize it
		adjustToolbar: function(editorName, initial){
			var stack = editorStack[editorName],
				self = this,
				elem = self.element;

			if(!stack.rightEditor || !stack.leftEditor){
				return;
			}
			var oldToolbarHeight = stack.toolbar.height(),
				toolbarWidth = (stack.rightEditor.outerWidth(true) + stack.rightEditor.offset().left) - stack.leftEditor.offset().left;

			// render toolbar with new values if toolbar flag is set
			!self.options.noToolbar && ep(stack.toolbar).css({
					'width' : toolbarWidth + 'px',
					'z-index': 1,
					display : ''
				});

			// calculate new toolbar height
			var newToolbarHeight = stack.toolbar.is(':visible') ? stack.toolbar.height() : 0,
				currentEditorNode = $(elem.ckeditorGet().container.$);

			// if height has changed or it is the first call -> set top padding
			if(oldToolbarHeight != newToolbarHeight || initial){
				for(var i = 0, iLength = stack.editors.length; i < iLength; i++){
					$(stack.editors[i].ckeditorGet().container.$).parent().css({
						'paddingTop' : newToolbarHeight+'px'
					});
				}
			}

			$(stack.toolbar).css({
				'position': 'absolute',
				'margin-top' : '-' + newToolbarHeight + 'px'
			});

			// set dimensions for disable layer
			if(stack.disabledLayer){
				stack.disabledLayer
					.width(toolbarWidth)
					.height(newToolbarHeight + $(elem.ckeditorGet().container.$).height());
			}

		},

		toggleToolbar: function(show){
			var self = this,
				o = self.options ;

			// move in another threat to ensure that hide/show is really called
			window.setTimeout(function(){
				editorStack[o.name].toolbar[show ? 'show' : 'hide']();
			}, 1);
			self.adjustToolbar(o.name,true);

		},

		destroy: function(){
			var self = this,
				o = self.options,
				name = o.name,
				destroy = true,
				stack = editorStack[name];

			self.Inherited(arguments);

			// check if all similar editor instances are undefined
			for(var key in stack.editors){
				if(stack.editors[key].data('epUiEditor')!==undefined){
					destroy = false;
				}
			}

			// if all similar editor instances are undefined
			if(destroy){
				// destroy all instances
				for(var key in stack.editors){
					CKEDITOR.instances[stack.editors[key].attr('id')].destroy();
				}
				// remove according toolbar
				stack.toolbar.remove();
				delete stack;
			}
		}
	});

});